// 函数mystrcat(char* dest, const char* src)的验证
//要点：
    //1.形参dest和src是字符串指针，其中dest应为字符数组。src指向的字符串在函数中不被修改。
    //2.字符数组dest能够容纳的字符数必须大于m+n。
    //3.函数把src所指字符串拼接到dest结尾处（覆盖dest结尾处的'\0'）并在串尾添加'\0'。
//关键字：字符串、内置谓词、幽灵代码、逻辑变量、循环变式

///////////////////////////////////////逻辑变量//////////////////////////////////////////////////
//@ logic int m, n;                 // m和n分别记录物理字符串dest和src的长度
//@ logic char *olddest, *oldsrc;   // olddest和oldsrc分别记录dest和src的在函数入口的值

/////////////////////////////////////////strcat函数的验证////////////////////////////////////////
//函数协议：
// 前条件：函数入口处dest和src的内容分别构成长度为m和n的物理字符串。字符数组dest的长度须大于m+n。
//         olddest和oldsrc分别记录dest和src的在函数入口的值。
// 后条件：函数返回值（\result）是一个指向长度为m+n的字符串指针，与函数入口处的dest相同。
//         生成的字符串（\result）是函数入口处dest和src所指向的字符串（分别为olddest和oldsrc）的拼接。   

/*@ 
  requires 
      \is_pstring(dest, m) && \is_pstring(src, n) 
      && m + n < \length(dest) - \offset(dest)   
      && olddest == dest == \old(dest) && oldsrc == src
      && \old(olddest[0..\length(olddest) - \offset(olddest) - 1]) == 
         olddest[0..\length(olddest) - \offset(olddest) - 1]
      ;
  assigns \old(dest)[0..\length(\old(dest)) - \offset(\old(dest)) - 1];
  ensures
      \is_pstring(\result, m+n) && \result == olddest 
      && \string(\result, m+n) == \string(olddest, m) + \string(oldsrc, n)
      && \is_pstring(oldsrc, n)
      && olddest[0..m-1] == \old(olddest[0..m-1])
      ;
*/
char* mystrcat(char* dest, const char* src) {
    char* cp = dest;
    //@ ghost int m1, n1;  // 幽灵变量

    //@ ghost m1 = 0;
    /*@    
      loop invariant 
          0 <= m1 <= m && cp == dest + m1 && (*cp == '\0' ==> m == m1) 
          && \is_pstring(olddest, m) && \is_pstring(oldsrc, n) 
          && m + n < \length(dest) - \offset(dest) 
          && \old(dest) == dest == olddest && src == oldsrc
          && \old(olddest[0..\length(olddest) - \offset(olddest) - 1]) == 
             olddest[0..\length(olddest) - \offset(olddest) - 1]
         ;
      loop variant m - m1;  // 循环变式
    */
    while(*cp != '\0') {
        cp = cp+1;
        //@ ghost m1 = m1 + 1;
    }
    *cp = *src;
    //@ ghost n1 = 0;
    /*@ 
      loop invariant 
          0 <= n1 <= n && cp == dest + m + n1 
          && *cp == *src && (*cp == '\0' ==> n == n1) 
          && \is_string(olddest, m+n1) 
          && \string(olddest, m+n1) == \string(olddest, m) + \string(oldsrc, n1) 
          && \is_pstring(oldsrc, n) && m + n < \length(dest) - \offset(dest) 
          && \old(dest) == dest == olddest && src == oldsrc + n1
          && olddest[0..m-1] == \old(olddest[0..m-1])
          ;
      loop variant n - n1;
    */    
    while(*cp != '\0') {
        //@ ghost n1 = n1 + 1;
        *(++cp) = *(++src);
    }
    return dest;
}